clear all
clc
% fault calculation program
%B. F. Wollenberg
% August 2008

j = sqrt(-1);
%Phase to Sequence and Sequence to Phase conversion Matrices
rad2deg = 180./3.14159;

deg2rad = 3.14159/180.;
a = cos(120*deg2rad) + j*sin(120*deg2rad);
a2 = a * a;

% Set up the A matrix and its inverse

A = [ 1  1  1
      1  a2 a 
      1  a  a2];  % Iabc = A*I012
  

% data file
NetworkData;


print_diagnostics = 0; % =0 do not print diagnostics, =1 print all diagnostics

if print_diagnostics == 1
	linedata
	busname

    Sbase
    Vbase1_LL
    Vbase2_LL

    Ibase1
    Ibase2

end


% build positive and negative sequence impedance matrices

Yzero = zeros(numbuses,numbuses);
Ypos = zeros(numbuses,numbuses);
Yneg = zeros(numbuses,numbuses);

%BUILD ZERO SEQUENCE Y MATRIX
for iline = 1:numlines
    
    switch trtype(iline)
        
    case { 0,1,3 } % trans line, Y-Y or generator internal impedance
        Yzero(fbus(iline),fbus(iline)) = Yzero(fbus(iline),fbus(iline)) + 1/(r0(iline) + j*x0(iline));
        Yzero(tbus(iline),tbus(iline)) = Yzero(tbus(iline),tbus(iline)) + 1/(r0(iline) + j*x0(iline));
        Yzero(fbus(iline),tbus(iline)) = Yzero(fbus(iline),tbus(iline)) - 1/(r0(iline) + j*x0(iline));
        Yzero(tbus(iline),fbus(iline)) = Yzero(tbus(iline),fbus(iline)) - 1/(r0(iline) + j*x0(iline));

        
%     case { 2 } % Y-D with Y on from bus
%         Yzero(fbus(iline),fbus(iline)) = Yzero(fbus(iline),fbus(iline)) + 1/(r0(iline) + j*x0(iline)); % Zzero to ground
%
    case { 2 } % D-Y with D on from bus
        Yzero(tbus(iline),tbus(iline)) = Yzero(tbus(iline),tbus(iline)) + 1/(r0(iline) + j*x0(iline)); % Zzero to ground
     
    end
end

% BUILD POSITIVE SEQUENCE Y MATRIX
for iline = 1:numlines
    switch trtype(iline)
       
    case { 0,1,3 } % trans line, Y-Y or generator internal impedance
        Ypos(fbus(iline),fbus(iline)) = Ypos(fbus(iline),fbus(iline)) + 1/(r1(iline) + j*x1(iline));
        Ypos(tbus(iline),tbus(iline)) = Ypos(tbus(iline),tbus(iline)) + 1/(r1(iline) + j*x1(iline));
        Ypos(fbus(iline),tbus(iline)) = Ypos(fbus(iline),tbus(iline)) - 1/(r1(iline) + j*x1(iline));
        Ypos(tbus(iline),fbus(iline)) = Ypos(tbus(iline),fbus(iline)) - 1/(r1(iline) + j*x1(iline));
        
    case { 2 } % Y-D with Y on from bus
        Ypos(fbus(iline),fbus(iline)) = Ypos(fbus(iline),fbus(iline)) + 1/(r1(iline) + j*x1(iline));
        Ypos(tbus(iline),tbus(iline)) = Ypos(tbus(iline),tbus(iline)) + 1/(r1(iline) + j*x1(iline));
        Ypos(fbus(iline),tbus(iline)) = Ypos(fbus(iline),tbus(iline)) - 1/(r1(iline) + j*x1(iline));
        Ypos(tbus(iline),fbus(iline)) = Ypos(tbus(iline),fbus(iline)) - 1/(r1(iline) + j*x1(iline));
                
    end
    
end

% BUILD NEGATIVE SEQUENCE Y MATRIX
for iline = 1:numlines
    switch trtype(iline)
        
    case { 0,1,3 } % trans line, Y-Y or generator internal impedance
        Yneg(fbus(iline),fbus(iline)) = Yneg(fbus(iline),fbus(iline)) + 1/(r1(iline) + j*x1(iline));
        Yneg(tbus(iline),tbus(iline)) = Yneg(tbus(iline),tbus(iline)) + 1/(r1(iline) + j*x1(iline));
        Yneg(fbus(iline),tbus(iline)) = Yneg(fbus(iline),tbus(iline)) - 1/(r1(iline) + j*x1(iline));
        Yneg(tbus(iline),fbus(iline)) = Yneg(tbus(iline),fbus(iline)) - 1/(r1(iline) + j*x1(iline));
        
    case { 2 } % Y-D with Y on from bus
        Yneg(fbus(iline),fbus(iline)) = Yneg(fbus(iline),fbus(iline)) + 1/(r1(iline) + j*x1(iline));
        Yneg(tbus(iline),tbus(iline)) = Yneg(tbus(iline),tbus(iline)) + 1/(r1(iline) + j*x1(iline));
        Yneg(fbus(iline),tbus(iline)) = Yneg(fbus(iline),tbus(iline)) - 1/(r1(iline) + j*x1(iline));
        Yneg(tbus(iline),fbus(iline)) = Yneg(tbus(iline),fbus(iline)) - 1/(r1(iline) + j*x1(iline));
        
    end
end


refbustype = 3;
for busi = 1:numbuses
    for busj = 1:numbuses
        if bustype(busi) == refbustype | bustype(busj) == refbustype
            Yzero(busi, busj ) = 0.0;
            Ypos(busi, busj ) = 0.0;
            Yneg(busi, busj ) = 0.0;
        end
    end
end

for busi = 1:numbuses
    if bustype(busi) == 3
        refbus = busi;
		Yzero(refbus, refbus) = 1.0;
		Ypos(refbus, refbus) = 1.0;
		Yneg(refbus, refbus) = 1.0;
    end
end

if print_diagnostics == 1
	Yzero
	Ypos
	Yneg
end

% test_zero = Yzero - conj(Yzero')
% test_pos = Ypos - conj(Ypos')
% test_neg = Yneg - conj(Yneg')

Zzero =inv(Yzero);
Zpos = inv(Ypos);
Zneg = inv(Yneg);

if print_diagnostics == 1
	Zii_zero = Zzero(faultbus,faultbus)
	Zii_pos =  Zpos(faultbus,faultbus)
	Zii_neg =  Zneg(faultbus,faultbus)
end

% assume that the 22 kV generator internal voltage is 1.0 per unit


%for fault_type = 0:1  % =1 is SGL  =0 is 3Phase

if fault_type == 2    %Single Line to Ground if fault_type = 2
	%----------------------------------------------------------------------
	%start single phase to ground fault
	% sequence current is 1.0 pu v / (Z0faultbus + Z+faultbus + Z-faultbus)
    fprintf(' \n')
    fprintf(' \n')
    fprintf('--------------------------- \n')
	fprintf('SINGLE LINE TO GROUND FAULT \n')
    fprintf('--------------------------- \n')
	Ifault = -1.0/(Zzero(faultbus,faultbus) + Zpos(faultbus,faultbus) + Zneg(faultbus,faultbus) + 3*Zfault);
	Ifault_abs = abs(Ifault)*Ibase2;
	
	Izero = zeros(numbuses,1);
           
	Ipos  = zeros(numbuses,1);
           
	Ineg  = zeros(numbuses,1);
          
    Izero(faultbus) = Ifault;
    Ipos(faultbus) =  Ifault;
    Ineg(faultbus) =  Ifault;
           
	%end single line to ground fault
    
else
    
	%----------------------------------------------------------------------
	% Symmetric Three Phase Fault calc at Fault Bus
    fprintf(' \n')
    fprintf(' \n')
    fprintf('--------------------------- \n')
	fprintf('THREE PHASE FAULT \n')
    fprintf('--------------------------- \n')
	Ifault = -1.0/(Zpos(faultbus,faultbus) + Zfault);
	Ifault_abs = abs(Ifault)*Ibase2;
    
	Izero = zeros(numbuses,1);   
	
	Ipos  = zeros(numbuses,1);    
	
	Ineg  = zeros(numbuses,1);     
     
    Ipos(faultbus) = Ifault;

	%end Three Phase Fault logic
	%----------------------------------------------------------------------
end


 Vint = ones(numbuses,1);  

 Vzero =       Zzero * Izero;
 
 Vpos = Vint + Zpos * Ipos;
 
 Vneg =        Zneg * Ineg;
 
 
 for ibus = 1:numbuses
     V012 = [ Vzero(ibus)
              Vpos(ibus)
              Vneg(ibus)];
     Vabc = A*V012;
     Vbus_abc(ibus,1)=Vabc(1);
     Vbus_abc(ibus,2)=Vabc(2);
     Vbus_abc(ibus,3)=Vabc(3);
     
 end
 
 if print_diagnostics == 1
	Vzero
	Vpos
	Vneg
 	
	Vbus_abc
    
end
 
 for ibus = 1:numbuses
     
     % change base so display is in kV
     switch ibus
         case{ 4,5,6,7 }
             Vbase_LN = Vbase1_LN/10^3;
             Vbase_LL = Vbase1_LL/10^3;
         case{ 1,2,3 }
             Vbase_LN = Vbase2_LN/10^3;
             Vbase_LL = Vbase2_LL/10^3;
     end
        
     Vabs_abc(ibus,1) = abs(Vbus_abc(ibus,1))*Vbase_LN;
     Vreal_a = real( Vbus_abc(ibus,1) );
     Vimag_a = imag( Vbus_abc(ibus,1) );
     Vangle_abc(ibus,1) = (atan2(Vimag_a, Vreal_a)) * rad2deg;
     
     Vabs_abc(ibus,2) = abs(Vbus_abc(ibus,2))*Vbase_LN;
     Vreal_b = real( Vbus_abc(ibus,2) );
     Vimag_b = imag( Vbus_abc(ibus,2) );
     Vangle_abc(ibus,2) = (atan2(Vimag_b, Vreal_b)) * rad2deg;
     
     Vabs_abc(ibus,3) = abs(Vbus_abc(ibus,3))*Vbase_LN;
     Vreal_c = real( Vbus_abc(ibus,3) );
     Vimag_c = imag( Vbus_abc(ibus,3) );
     Vangle_abc(ibus,3) = (atan2(Vimag_c, Vreal_c)) * rad2deg;
      
	 VLL_AB(ibus) = ( abs( Vbus_abc(ibus,1) - Vbus_abc(ibus,2))/sqrt(3) ) * Vbase_LL;
	 VLL_BC(ibus) = ( abs( Vbus_abc(ibus,2) - Vbus_abc(ibus,3))/sqrt(3) ) * Vbase_LL;
	 VLL_CA(ibus) = ( abs( Vbus_abc(ibus,3) - Vbus_abc(ibus,1))/sqrt(3) ) * Vbase_LL;
     
     
 end
 

fprintf('%s  %8.2f %s \n','   22 kV Line-Line the Line-Neutral Voltage is  ', 22/sqrt(3),  ' kV')
fprintf('%s  %8.2f %s \n',' 345 kV Line-Line the Line-Neutral Voltage is  ', 345/sqrt(3),' kV')

fprintf(' \n')
fprintf(' Line to Neutral Phase voltages at each bus \n');
for ibus = 1:numbuses
  fprintf( ' %s  %3d   %s   %s   %8.2f    %s   %8.2f   %s   %8.2f \n',...
                      ' Bus ',ibus,  busname(ibus,:), ' Va =    ', Vabs_abc(ibus,1),...
                                                      ' Vb =    ', Vabs_abc(ibus,2),...
                                                      ' Vc =    ', Vabs_abc(ibus,3) );
  
  VLLab = VLL_AB(ibus);
  VLLbc = VLL_BC(ibus);
  VLLca = VLL_CA(ibus);
  fprintf( ' %s             %s   %8.2f    %s   %8.2f   %s   %8.2f \n',...
                               '                    ', ' VLLab = ', VLL_AB(ibus),...
                                                       ' VLLbc = ', VLL_BC(ibus),...
                                                       ' VLLca = ', VLL_CA(ibus) );                                                              


end
fprintf( ' \n')

	for ibus = 1:numbuses
              fprintf( ' %s  %3d   %s   %s   %10.4f    %s   %10.4f   %s   %10.4f  %s   %10.4f    %s   %10.4f   %s   %10.4f \n',...
                                  ' Bus ',ibus,  busname(ibus,:), ' Va = ', Vabs_abc(ibus,1),' Vangle = ', Vangle_abc(ibus,1),...
                                                                  ' Vb = ', Vabs_abc(ibus,2),' Vangle = ', Vangle_abc(ibus,2),...
                                                                  ' Vc = ', Vabs_abc(ibus,3),' Vangle = ', Vangle_abc(ibus,3) );
	end
	fprintf( ' \n')
 



 % calculate sequence currents
 for iline = 1:numlines
         Izero_fromend(iline) = 0;
         Izero_toend(iline)   = 0;
         
         Ipos_fromend(iline) = 0;
         Ipos_toend(iline)   = 0;  
         
         Ineg_fromend(iline) = 0;
         Ineg_toend(iline)   = 0;
     end
 
 for iline = 1:numlines
     switch trtype(iline)
 
     case { 0,1,3 } % line
         Izero_fromend(iline) = (Vzero(fbus(iline)) - Vzero(tbus(iline)))/(r0(iline) + j*x0(iline));
         Izero_toend(iline)   = (Vzero(tbus(iline)) - Vzero(fbus(iline)))/(r0(iline) + j*x0(iline));
         
         Ipos_fromend(iline) = (Vpos(fbus(iline)) - Vpos(tbus(iline)))/(r1(iline) + j*x1(iline));
         Ipos_toend(iline)   = (Vpos(tbus(iline)) - Vpos(fbus(iline)))/(r1(iline) + j*x1(iline));  
         
         Ineg_fromend(iline) = (Vneg(fbus(iline)) - Vneg(tbus(iline)))/(r1(iline) + j*x1(iline));
         Ineg_toend(iline)   = (Vneg(tbus(iline)) - Vneg(fbus(iline)))/(r1(iline) + j*x1(iline));
         
     case { 2 } % Y-D transformer   
         
         Izero_fromend(iline) = (Vzero(fbus(iline)))/(r0(iline) + j*x0(iline));
         Izero_toend(iline)   = 0.0;
         
         Ipos_fromend(iline) = (Vpos(fbus(iline)) - Vpos(tbus(iline)))/(r1(iline) + j*x1(iline));
         Ipos_toend(iline)   = (Vpos(tbus(iline)) - Vpos(fbus(iline)))/(r1(iline) + j*x1(iline));  
         
         Ineg_fromend(iline) = (Vneg(fbus(iline)) - Vneg(tbus(iline)))/(r1(iline) + j*x1(iline));
         Ineg_toend(iline)   = (Vneg(tbus(iline)) - Vneg(fbus(iline)))/(r1(iline) + j*x1(iline));         
         
          
     end
 end

% PRINT CURRENTS

fprintf(' %s   %3d  \n\n',' fault bus = ',faultbus)
 
for ibus = 1:numbuses
    fprintf( ' \n')
    fprintf( '%s \n','------------------------------------------------------')
    fprintf( '%s  %2d  %s  \n', 'Bus ',ibus, busname(ibus,:));


    for iline = 1:numlines
         
        switch trtype(iline)
            
        case { 0 } % transmission line at 345 kV
            Ibase_from = Ibase2; % from end is at 345 kV
            Ibase_to = Ibase2;   % to end is at 345 kV
            
        case { 1,2 } % transformers with primary on from end at 22 kV secondary on to end at 345 kV
            Ibase_from = Ibase1; % from end is at 22 kV
            Ibase_to = Ibase2;   % to end is at 345 kV

        case { 3 }   % generator equiv impedance at 22 kV both ends
            Ibase_from = Ibase1; % from end is at 22 kV
            Ibase_to = Ibase1; % from end is at 22 kV

        end
        
    if ibus == fbus(iline)
             
             fprintf( ' \n')
             fprintf( '%s \n  %s  %2d \n',branchname(iline,:), 'at bus ', fbus(iline) );
             
             sign_current_zero = 1*sign( real( Izero_fromend(iline) ) );
             sign_current_pos  = 1*sign( real( Ipos_fromend(iline) ) ) ;
             sign_current_neg  = 1*sign( real( Ineg_fromend(iline) ) ) ;

             fprintf( '%s   %6.0f \n','         I zero =', sign_current_zero*abs( Izero_fromend(iline) ) * Ibase_from );
             fprintf( '%s   %6.0f \n','         I pos  =', sign_current_pos *abs( Ipos_fromend(iline)  ) * Ibase_from );
             fprintf( '%s   %6.0f \n','         I neg  =', sign_current_neg *abs( Ineg_fromend(iline)  ) * Ibase_from );
             
             I012_fromend = [ Izero_fromend(iline)
                              Ipos_fromend(iline)
                              Ineg_fromend(iline) ] * Ibase_from;
             Iabc_fromend = A*I012_fromend;
             
% 		     sign_current(1) = 1*sign( real( Vbus_abc(ibus,1)*conj(Iabc_fromend(1)) ) );
%            sign_current(2) = 1*sign( real( Vbus_abc(ibus,2)*conj(Iabc_fromend(2)) ) );
%            sign_current(3) = 1*sign( real( Vbus_abc(ibus,3)*conj(Iabc_fromend(3)) ) );
             
		     sign_current(1) = 1*sign( real( Iabc_fromend(1)) );
             sign_current(2) = 1*sign( real( Iabc_fromend(2)) );
             sign_current(3) = 1*sign( real( Iabc_fromend(3)) );   
 

             fprintf( '%s   %6.0f  %6.0f  %6.0f  \n ','         I abc  =', sign_current'.*abs(Iabc_fromend) );

    
    elseif ibus == tbus(iline)
    
             fprintf( ' \n')
             fprintf( '%s \n %s  %2d  \n',branchname(iline,:),'at bus ', tbus(iline) );
             
             sign_current_zero = 1*sign( real( Izero_toend(iline)) ) ;
             sign_current_pos  = 1*sign( real( Ipos_toend(iline)) ) ;
             sign_current_neg  = 1*sign( real( Ineg_toend(iline)) ) ;

             fprintf( '%s   %6.0f  \n','         I zero =', sign_current_zero*abs( Izero_toend(iline)   ) * Ibase_to );
             fprintf( '%s   %6.0f  \n','         I pos  =', sign_current_pos*abs( Ipos_toend(iline)    ) * Ibase_to );
             fprintf( '%s   %6.0f  \n','         I neg  =', sign_current_neg*abs( Ineg_toend(iline)    ) * Ibase_to );
             
             I012_toend = [ Izero_toend(iline)
                            Ipos_toend(iline)
                            Ineg_toend(iline) ] * Ibase_to;
             Iabc_toend = A*I012_toend;
             
% 		     sign_current(1) = 1*sign( real( Vbus_abc(ibus,1)*conj(Iabc_toend(1)) ) );
%            sign_current(2) = 1*sign( real( Vbus_abc(ibus,2)*conj(Iabc_toend(2)) ) );
%            sign_current(3) = 1*sign( real( Vbus_abc(ibus,3)*conj(Iabc_toend(3)) ) );

		     sign_current(1) = 1*sign( real( Iabc_toend(1)) );
             sign_current(2) = 1*sign( real( Iabc_toend(2)) );
             sign_current(3) = 1*sign( real( Iabc_toend(3)) );   
             
             
             fprintf( '%s   %6.0f  %6.0f  %6.0f  \n ','         I abc  =', sign_current'.*abs(Iabc_toend) );

    end % end of if ibus
    
    end % end of for loop on ilines
    % print fault current
    if ibus == faultbus
        Ifault_seq = [ Izero(faultbus)
                       Ipos(faultbus)
                       Ineg(faultbus) ];
        Ifault_abc = A*Ifault_seq;
        fprintf( ' \n')
    fprintf( '%s \n','------------------------------------------------------')
        fprintf( ' \n')
        fprintf( '%s \n','****************************************************')
        fprintf( '%s  %2d  %s  \n', 'FAULT at Bus ',ibus, busname(ibus,:));
        fprintf( '%s   %6.0f %6.0f  %6.0f  \n','Fault current   =', abs( Ifault_abc ) * Ibase2 ); 
        fprintf( '%s \n','****************************************************')
    end
    
end % end of for loop on buses
 
%end % end loop on fault_type 
 
