
function [Y Z Zc] = inputBusData(CAPFLAG)

%---------Base Values ---------%
kVLL=345;
MVA3Ph=100;
Zbase=kVLL^2/MVA3Ph;

XL_km=0.376;                        % ohm/km at 60 Hz
RL_km= 0.037; B_km=4.5;             % B in micro-mho/km

% -------- Bus lengths ---------%

busLength = ones(3,3);
busLength(1,3) = 200;
busLength(3,1) = busLength(1,3);
busLength(1,2) = 150;
busLength(2,1) = busLength(1,2);
busLength(2,3) = 150;
busLength(3,2) = busLength(2,3);

%-------- impedance matrix --------%
Z = zeros(3,3);
Zc = zeros(3,3);
for i = 1:3
    for j = 1:3
        Z(i,j) = (RL_km+(sqrt(-1))*XL_km)*busLength(i,j)/Zbase;          % resistive impedance
        Zc(i,j) = -2*sqrt(-1)*inv(B_km*busLength(i,j)*Zbase*10^-6);    % capacitive impedance
    end
end


busLength = ones(3,3);
for i = 1:3
       Z(i,i) = inf;
       Zc(i,i) = inf;
end;


% --------- admittance matrix ------------ %
Y = zeros(3,3);
if(CAPFLAG)
    for i = 1:length(Z)
        for j = 1:length(Z)
            if(i~=j)
                Y(i,j) = -inv(Z(i,j));
            else
                Y(i,j) = sum(1./(Z(:,j))) + sum(1./(Zc(:,j)));  % with capacitive admittance
            end;
        end;
    end;
else
    for i = 1:length(Z)
        for j = 1:length(Z)
            if(i~=j)
                Y(i,j) = -inv(Z(i,j));
            else
                Y(i,j) = sum(1./Z(:,j));                      % without capacitive admittance
            end;
        end;
    end;
end;

return