
function [P Q] = runGaussSeidel(v, angle, Psp, Qsp, CAPFLAG)

MAX_ITER = 20;
iter = 1;
convFlag = 1;
TOL  = 0.001;

% ------- bus type and admittance ---------- %

[Y Z Zc] = inputBusData(CAPFLAG);
Bustype(1) = 'S';                          % PQ bus
Bustype(2) = 'G';                          % PV bus
Bustype(3) = 'L';                          % PQ bus


% ------- Initialize line variables --------- %

I = zeros(1,length(Z));
P = zeros(length(Z),length(Z));
Q = zeros(length(Z),length(Z));
S = zeros(1,length(Z));


% -------- Input bus parameters --------- %

Psched = zeros(1,length(Z));
Qsched = zeros(1,length(Z));

% ------ Set up power schedule --------- %

for i = 1:length(Z)
    Psched(i)= Psp(i);
    Qsched(i)= Qsp(i);
end

% --- Calculate P and Q at each bus ----- %

for i = 1:length(Z)
	I(i) = 0.0 + sqrt(-1)*0.0;
        for j = 1:length(Z)
            I(i) = I(i) + Y(i,j)*v(j);
	end;
        S(i) = v(i) * conj(I(i));
	if(CAPFLAG)
		 for j = 1:length(Z)
            		if(i~=j)
                		P(i,j) = real(v(i)*conj(v(i) - v(j)/Z(i,j)) + v(i)/Zc(i,j));
                		Q(i,j) = imag(v(i)*conj(v(i) - v(j)/Z(i,j)) + v(i)/Zc(i,j));
            		else
                		P(i,j) = real(S(i));
                		Q(i,j) = imag(S(i));
            		end;
        	end;
	else
		for j = 1:length(Z)
	            if(i~=j)
        	        P(i,j) = real(v(i)*conj(v(i) - v(j)/Z(i,j)));
                	Q(i,j) = imag(v(i)*conj(v(i) - v(j)/Z(i,j)));
            	    else
            	        P(i,j) = real(S(i));
                	Q(i,j) = imag(S(i));
            	    end;
        	end;
	end;
end;


    while(convFlag==1 && iter < MAX_ITER)
        MAXDP = 0.0;
        MAXDQ = 0.0;

        for i = 1 : length(Z)

            %---------calculate net P and Q at bus i--------%

            I(i) = 0.0 + sqrt(-1)*0.0;
            for j = 1:length(Z)
                I(i) = I(i) + Y(i,j)*v(j);
            end
            S(i) = v(i) * conj(I(i));
            if(CAPFLAG)
		 for j = 1:length(Z)
            		if(i~=j)
                		P(i,j) = real(v(i)*conj(v(i) - v(j)/Z(i,j)) + v(i)/Zc(i,j));
                		Q(i,j) = imag(v(i)*conj(v(i) - v(j)/Z(i,j)) + v(i)/Zc(i,j));
            		else
                		P(i,j) = real(S(i));
                		Q(i,j) = imag(S(i));
            		end;
        	end;
	    else
		for j = 1:length(Z)
	            if(i~=j)
        	        P(i,j) = real(v(i)*conj(v(i) - v(j)/Z(i,j)));
                	Q(i,j) = imag(v(i)*conj(v(i) - v(j)/Z(i,j)));
            	    else
            	        P(i,j) = real(S(i));
                	Q(i,j) = imag(S(i));
            	    end;
        	end;
	    end;


            if Bustype(i) == 'G'
                Qsched(i) = Q(i,i);
            end
            deltap(i) = abs(P(i,i) - Psched(i));
            deltaq(i) = 0.0;

            if Bustype(i) == 'L'
                deltaq(i) = abs(Q(i,i) - Qsched(i));
            end

            if Bustype(i) == 'S'
                deltap(i) = 0.0;
                deltaq(i) = 0.0;
            end

            if Bustype(i) ~= 'S'
                if deltap(i) > MAXDP
                    MAXDP = deltap(i);
                end

                if deltaq(i) > MAXDQ
                    MAXDQ = deltaq(i);
                end

                %--------- Y * V for row i of Y matrix without Yii term---------%

                sum = 0.0;
                for j = 1:length(Z)
                    if j ~= i
                        sum = sum + Y(i,j)*v(j);
                    end
                end
                vnew = (1.0/Y(i,i))*( ((Psched(i) - sqrt(-1)*Qsched(i))/(conj(v(i)))) - sum);
                v(i) = vnew;
            end
        end

        %------------ Print and save result from last iteration-----------%

        %-----calculate net P and Q at bus i-----%

        for i = 1:length(Z)
            I(i) = 0.0 + sqrt(-1)*0.0;
            for j = 1:length(Z)
                I(i) = I(i) + Y(i,j)*v(j);
            end
            S(i) = v(i) * conj(I(i));
           if(CAPFLAG)
		 for j = 1:length(Z)
            		if(i~=j)
                		P(i,j) = real(v(i)*conj(v(i) - v(j)/Z(i,j)) + v(i)/Zc(i,j));
                		Q(i,j) = imag(v(i)*conj(v(i) - v(j)/Z(i,j)) + v(i)/Zc(i,j));
            		else
                		P(i,j) = real(S(i));
                		Q(i,j) = imag(S(i));
            		end;
        	end;
	   else
		for j = 1:length(Z)
	            if(i~=j)
        	        P(i,j) = real(v(i)*conj(v(i) - v(j)/Z(i,j)));
                	Q(i,j) = imag(v(i)*conj(v(i) - v(j)/Z(i,j)));
            	    else
            	        P(i,j) = real(S(i));
                	Q(i,j) = imag(S(i));
            	    end;
        	end;
	  end;


            if Bustype(i) == 'G'
                Qsched(i) = Q(i,i);
            end
        end

        displayResults(P,Q,v,iter);

        % -------- check for convergence ---------- %

        if MAXDP < TOL
            if MAXDQ < TOL
                convFlag = 0;
            end
        end
        iter = iter + 1;
    end
    return;

